import React, {
    useContext
} from "react";

import {
    getChampionshipPosition,
    getPointsAheadMax,
    getPointsAheadMin,
    getPointsAvailableForRace,
    getPointsAvailableFromRace,
    getRacesLeft
} from "../utility/champCalculator";
import {
    YearContext
} from "../pages/YearRoute";
import CountryRaceName from "./CountryRaceName";

function DriverComparisonTable({ driver1, driver2 }) {
    const {
        pointsMultiTeam,
        year,
        points,
    } = useContext(YearContext);
    if(!pointsMultiTeam) {
        return null;
    }

    const racesLeft = getRacesLeft(year);
    const driver1Position = getChampionshipPosition(year, driver1);
    const driver1Points = pointsMultiTeam.drivers[driver1];
    const driver2Points = pointsMultiTeam.drivers[driver2];

    const pointsFromRace = getPointsAvailableFromRace(year, points);
    const isPosition = (driver1Points - driver2Points) > pointsFromRace

    return (
        <table className="table table-bordered">
            <thead>
            <tr>
                <th>Race</th>
                <th className="text-end d-none d-md-table-cell">Max points</th>
                <th className="text-end">Difference after { driver2 } no points</th>
                <th className="text-end">Difference after { driver2 } P2 finish</th>
                <th className="text-end">Points left after</th>
                <th className="d-none d-md-table-cell">Possible outcome</th>
            </tr>
            </thead>
            <tbody>
            <tr className={ isPosition ? "table-success" : "" }>
                <td>Current</td>
                <td className="d-none d-md-table-cell"/>
                <td/>
                <td className="text-end">{ driver1Points - driver2Points }</td>
                <td className="text-end">{ pointsFromRace }</td>
                <td className="d-none d-md-table-cell">
                    { isPosition && (
                        driver1Position === 1 ? (
                            driver1 + " is champion"
                        ) : (
                            driver1 + " is P" + driver1Position + " in championship"
                        )
                    )}
                </td>
            </tr>
            <tr className={ "d-md-none" + (isPosition ? "" : "table-success") }>
                { isPosition && (
                    driver1Position === 1 ? (
                        driver1 + " is champion"
                    ) : (
                        driver1 + " is P" + driver1Position + " in championship"
                    )
                )}
            </tr>
            { racesLeft.map((race, raceIndex) => {
                const racePoints = getPointsAvailableForRace(race, points);
                const pointsLeft = getPointsAvailableFromRace(year, points, race);
                const pointsAheadMax = getPointsAheadMax(year, points, driver1, driver2, race);
                const pointsAheadMin = getPointsAheadMin(year, points, driver1, driver2, race);
                const wouldBePosition = pointsAheadMax > pointsLeft;
                const couldBePosition = pointsAheadMin > pointsLeft;

                const outcome = isPosition ? (
                    driver1Position === 1 ? (
                        driver1 + " is champion"
                    ) : (
                        driver1 + " is P" + driver1Position + " in championship"
                    )
                ) : wouldBePosition ? (
                    driver1Position === 1 ? (
                        driver1 + " would be champion"
                    ) : (
                        driver1 + " would be P" + driver1Position + " in championship"
                    )
                ) : couldBePosition ? (
                    driver1Position === 1 ? (
                        driver1 + " could be champion"
                    ) : (
                        driver1 + " could be P" + driver1Position + " in championship"
                    )
                ) : (
                    "Not decided"
                );

                const rowClass = wouldBePosition ? "table-success" : (couldBePosition ? "table-warning" : "table-danger");

                return (
                    <React.Fragment key={ raceIndex }>
                        <tr
                            key={ raceIndex }
                            className={ rowClass }
                        >
                            <td>
                                <CountryRaceName
                                    country={ race.country }
                                    name={ race.name }
                                />
                            </td>
                            <td className="text-end d-none d-md-table-cell">{racePoints}</td>
                            <td className="text-end">{pointsAheadMin}</td>
                            <td className="text-end">{pointsAheadMax}</td>
                            <td className="text-end">{pointsLeft}</td>
                            <td className="d-none d-md-table-cell">
                                { outcome }
                            </td>
                        </tr>
                        <tr className={ `d-md-none ${rowClass}` }>
                            <td colSpan={ 4 }>
                                { outcome }
                            </td>
                        </tr>
                    </React.Fragment>
                )
            })}
            </tbody>
        </table>
    )
}

export default React.memo(DriverComparisonTable);
