import React, {
    useCallback,
    useMemo,
    useRef,
    useState
} from "react";
import {
    Container,
    Nav,
    Navbar as RBNavbar,
    NavDropdown
} from "react-bootstrap";
import {
    Link,
    useRouteMatch
} from "react-router-dom";

import {
    getData
} from "../pages/YearRoute";
import useOutsideAlerter from "../hooks/OutsideAlerter";

function NavbarLink({to, title, exact = false, onClick}) {
    let match = useRouteMatch({
        path: to,
        exact: exact
    });
    let className = "nav-item" + (match ? " text-primary" : "");
    return (
        <li className={ className } data-toggle="collapse" data-target=".navbar-collapse.show">
            <Link to={ to } className="nav-link" onClick={ onClick }>
                { title }
            </Link>
        </li>
    );
}

function DropdownLink({to, title, exact = false, onClick}) {
    let match = useRouteMatch({
        path: to,
        exact: exact
    });
    let className = "dropdown-item " + (match ? "active" : "");
    return (
        <Link to={ to } className={ className } onClick={ onClick }>{ title }</Link>
    )
}

function Navbar() {
    const [showCollapsed, setShowCollapsed] = useState(false);
    const collapse = useCallback(() => {
        setShowCollapsed(false);
    }, []);

    const [showYearDropdown, setShowYearDropdown] = useState(false);
    const dropdownRef = useRef(null);
    const toggleYearDropdown = useCallback(() => {
        setShowYearDropdown(!showYearDropdown);
    }, [showYearDropdown]);
    useOutsideAlerter(dropdownRef, () => setShowYearDropdown(false));

    const data = useMemo(() => {
        return getData();
    }, []);
    const match = useRouteMatch("/:year");
    const year = useMemo(() => {
        const matchYear = match ? match.params.year : null;
        if(!matchYear) {
            return null;
        }
        if(!Object.keys(data).includes(matchYear)) {
            return null;
        }
        return matchYear;
    }, [match, data]);

    return (
        <React.Fragment>
            <RBNavbar
                bg="dark"
                expand="lg"
                className="fixed-top"
                expanded={ showCollapsed }
                style={{ borderBottom: "#414141 1px solid" }}
            >
                <Container>
                    <a className="navbar-brand" href="/">
                        When F1 Champion { year }
                    </a>
                    <RBNavbar.Toggle aria-controls="basic-navbar-nav" onClick={ () => setShowCollapsed(!showCollapsed) } />
                    <RBNavbar.Collapse id="basic-navbar-nav">
                        <Nav className="flex-grow-1 d-flex justify-content-md-center">
                            {year && (
                                <React.Fragment>
                                    <NavbarLink to={`/${year}/races`} title="Races" onClick={ collapse }/>
                                    <NavbarLink to={`/${year}/drivers`} title="Drivers" onClick={ collapse }/>
                                    <NavbarLink to={`/${year}/teams`} title="Teams" onClick={ collapse }/>
                                </React.Fragment>
                            )}
                        </Nav>
                        <Nav>
                            <NavDropdown
                                ref={ dropdownRef }
                                title={ year ?? "Year" }
                                id="year-dropdown"
                                show={ showYearDropdown }
                                onClick={ toggleYearDropdown }
                            >
                                { Object.keys(data).map((year) => (
                                    <DropdownLink to={ `/${year}` } title={ year } onClick={ collapse } key={ year }/>
                                ))}
                            </NavDropdown>
                        </Nav>
                    </RBNavbar.Collapse>
                </Container>
            </RBNavbar>

            <div style={{ height: "56px" }}/>
        </React.Fragment>
    )
}

export default React.memo(Navbar);
