import React, {
    useContext
} from "react";

import {
    getPointsAvailableForRace,
    getRacesLeft,
    getTotalPointsLeft
} from "../utility/champCalculator";

import {
    YearContext
} from "./YearRoute";
import CountryRaceName from "../components/CountryRaceName";

function Races() {
    const {
        races,
        year,
        points,
    } = useContext(YearContext);
    const racesLeft = getRacesLeft(year);
    const pointsLeft = getTotalPointsLeft(year, points);
    const showTrack = races?.some((race) => race.track);
    const showCity = races?.some((race) => race.city);
    if(!races) {
        return (
            <div className="container mt-3"/>
        );
    }
    return (
        <div className="container mt-3">
            <h1>Races</h1>
            <p>
                <b>
                    Total points left to gain: { pointsLeft }
                </b>
            </p>
            <table className="table table-bordered">
                <thead>
                <tr>
                    <th/>
                    <th>Race</th>
                    { showTrack && (
                        <th className="d-none d-md-table-cell">Track</th>
                    )}
                    { showCity && (
                        <th className="d-none d-md-table-cell">City</th>
                    )}
                    <th>Points</th>
                </tr>
                </thead>
                <tbody>
                { races?.map((race, index) => {
                    const left = racesLeft.includes(race);
                    const nextUp = race === racesLeft[0];
                    let className = "table-success";
                    if(nextUp) {
                        className = "table-info";
                    } else if(left) {
                        className = "table-danger";
                    }
                    return (
                        <tr key={ index } className={ className }>
                            <td style={{ verticalAlign: "middle" }} className="text-center text-md-left">
                                { nextUp ? (
                                    <>
                                        <i className="fa-sharp fa-solid fa-clock fa-fw me-md-2"/>
                                        <span className="d-none d-md-inline-block">
                                            Next up
                                        </span>
                                    </>
                                ) : left ? (
                                    <>
                                        <i className="fa-sharp fa-solid fa-xmark fa-fw me-md-2"/>
                                        <span className="d-none d-md-inline-block">
                                            Remaining
                                        </span>
                                    </>
                                ) : (
                                    <>
                                        <i className="fa-sharp fa-solid fa-check fa-fw me-md-2"/>
                                        <span className="d-none d-md-inline-block">
                                            Done
                                        </span>
                                    </>
                                )}
                            </td>
                            <td style={{ verticalAlign: "middle" }}>
                                <CountryRaceName
                                    country={ race.country }
                                    name={ race.name }
                                />
                            </td>
                            { showTrack && (
                                <td style={{ verticalAlign: "middle" }} className="d-none d-md-table-cell">
                                    { race.track }
                                </td>
                            )}
                            { showCity && (
                                <td style={{ verticalAlign: "middle" }} className="d-none d-md-table-cell">
                                    { race.city }
                                </td>
                            )}
                            <td style={{ verticalAlign: "middle" }}>
                                { getPointsAvailableForRace(race, points) }
                            </td>
                        </tr>
                    )
                })}
                </tbody>
            </table>
        </div>
    )
}

export default React.memo(Races);
