import React, {
    useContext,
    useMemo
} from "react";
import {
    Link,
    useRouteMatch
} from "react-router-dom";

import {
    getTeamPoints
} from "../utility/champCalculator";
import {
    YearContext
} from "./YearRoute";

function TeamChip({ teamKey, team, points }) {
    const { year } = useContext(YearContext);
    const url = `/${year}/teams/${ teamKey }`;
    const match = useRouteMatch({
        path: url,
        exact: true
    });

    const className = useMemo(() => {
        let classNames = ["btn rounded-pill me-2 mb-2"];
        if (match) {
            classNames.push("btn-primary");
        } else {
            classNames.push("bg-danger-subtle border-danger text-danger-emphasis");
        }
        return classNames.join(" ")
    }, [match]);

    return (
        <Link
            to={ url }
            className={ className }
        >
            { team.name } { points }
        </Link>
    );
}

function Team({ match }) {
    const {
        teams,
        year
    } = useContext(YearContext);

    const sortedTeamPoints = useMemo(() => {
        if(!teams) {
            return [];
        }
        let teamPoints = {};
        for(const teamKey of Object.keys(teams)) {
            teamPoints[teamKey] = getTeamPoints(year, teamKey);
        }
        return Object.entries(teamPoints).sort((team1, team2) => {
            return team2[1] - team1[1];
        });
    }, [year, teams]);

    if(!teams) {
        return null;
    }

    const teamString = match.params.team;

    const teamData = teamString ? teams[teamString] : null;

    return (
        <div className="container mt-3">
            <div className="my-3">
                { sortedTeamPoints.map(([teamKey, points], index) => {
                    const team = teams[teamKey];
                    return (
                        <TeamChip
                            key={ index }
                            teamKey={ teamKey }
                            team={ team }
                            points={ points }
                        />
                    );
                })}
            </div>

            { !teamData ? (
                <div className="text-center">
                    <h2>
                        Select a team above
                    </h2>
                </div>
            ) : (
                <div className="text-center">
                    <h2>
                        { teamData.name }
                    </h2>
                </div>
            )}
        </div>
    );
}

export default React.memo(Team);
